@@ -119,7 +119,8 @@ def min_max(box, width):
         max_width = max_width.value
 
     if isinstance(box, boxes.ReplacedBox):
-        ratio = box.replacement.intrinsic_ratio
+        _, _, ratio = box.replacement.get_intrinsic_size(
+            1, box.style['font_size'])
         if ratio is not None:
             min_height = box.style['min_height']
             if min_height != 'auto' and min_height.unit != '%':
@@ -267,7 +268,7 @@ def inline_line_widths(context, box, outer, is_line_start, minimum,
     if skip_stack is None:
         skip = 0
     else:
-        skip, skip_stack = skip_stack
+        (skip, skip_stack), = skip_stack.items()
     for child in box.children[skip:]:
         if child.is_absolutely_positioned():
             continue  # Skip
@@ -291,7 +292,7 @@ def inline_line_widths(context, box, outer, is_line_start, minimum,
             if skip_stack is None:
                 skip = 0
             else:
-                skip, skip_stack = skip_stack
+                (skip, skip_stack), = skip_stack.items()
                 assert skip_stack is None
             child_text = child.text[(skip or 0):]
             if is_line_start and space_collapse:
@@ -301,18 +302,18 @@ def inline_line_widths(context, box, outer, is_line_start, minimum,
             else:
                 max_width = 0 if minimum else None
                 lines = []
-                resume_at = new_resume_at = 0
-                while new_resume_at is not None:
-                    resume_at += new_resume_at
-                    _, _, new_resume_at, width, _, _ = (
+                resume_index = new_resume_index = 0
+                while new_resume_index is not None:
+                    resume_index += new_resume_index
+                    _, _, new_resume_index, width, _, _ = (
                         split_first_line(
-                            child_text[resume_at:], child.style, context,
+                            child_text[resume_index:], child.style, context,
                             max_width, child.justification_spacing,
                             minimum=True))
                     lines.append(width)
                     if first_line:
                         break
-                if first_line and new_resume_at:
+                if first_line and new_resume_index:
                     current_line += lines[0]
                     break
         else:
@@ -656,11 +657,12 @@ def replaced_min_content_width(box, outer=True):
             width = 0
         else:
             image = box.replacement
-            iwidth, iheight = image.get_intrinsic_size(
-                box.style['image_resolution'], box.style['font_size'])
+            intrinsic_width, intrinsic_height, intrinsic_ratio = (
+                image.get_intrinsic_size(
+                    box.style['image_resolution'], box.style['font_size']))
             width, _ = default_image_sizing(
-                iwidth, iheight, image.intrinsic_ratio, 'auto', height,
-                default_width=300, default_height=150)
+                intrinsic_width, intrinsic_height, intrinsic_ratio, 'auto',
+                height, default_width=300, default_height=150)
     elif box.style['width'].unit == '%':
         # See https://drafts.csswg.org/css-sizing/#intrinsic-contribution
         width = 0
@@ -681,10 +683,11 @@ def replaced_max_content_width(box, outer=True):
             assert height.unit == 'px'
             height = height.value
         image = box.replacement
-        iwidth, iheight = image.get_intrinsic_size(
-            box.style['image_resolution'], box.style['font_size'])
+        intrinsic_width, intrinsic_height, intrinsic_ratio = (
+            image.get_intrinsic_size(
+                box.style['image_resolution'], box.style['font_size']))
         width, _ = default_image_sizing(
-            iwidth, iheight, image.intrinsic_ratio, 'auto', height,
+            intrinsic_width, intrinsic_height, intrinsic_ratio, 'auto', height,
             default_width=300, default_height=150)
     elif box.style['width'].unit == '%':
         # See https://drafts.csswg.org/css-sizing/#intrinsic-contribution
